searchData={"items":[{"type":"module","title":"grisp","doc":"GRiSP Runtime API.\n\nThis module contains the main API for working with the GRiSP runtime.","ref":"grisp.html"},{"type":"function","title":"grisp.add_device/2","doc":"","ref":"grisp.html#add_device/2"},{"type":"function","title":"grisp.add_device/3","doc":"Add and start the device instance for a connected device.\n\nThis starts a device driver instance of the specified module and connected to\nthe specified slot. No hardware validation is possible to check that the\ncorrect device is actually connected, but many drivers run an initialization\ncheck for specific device IDs and other similar characteristics which will\nfail if no device or the wrong device is connected.\n\nThe set of options is passed to the driver and is specific to each driver.\n\nReturns the created device instance.","ref":"grisp.html#add_device/3"},{"type":"function","title":"grisp.device/1","doc":"Return the device instance at the specified slot (if any).","ref":"grisp.html#device/1"},{"type":"function","title":"Errors - grisp.device/1","doc":"* `{no_device_connected, Slot}`.","ref":"grisp.html#device/1-errors"},{"type":"function","title":"grisp.devices/0","doc":"List all running devices instances.\n\nReturns a list of all running device instances as device objects.","ref":"grisp.html#devices/0"},{"type":"function","title":"grisp.remove_device/1","doc":"Remove and stop the device instance for a device.\n\nThis will gracefully stop the device instance and remove it from the runtime.\nThis includes running any termination code inside the device driver module\n(see the specific driver module for your device for more information).","ref":"grisp.html#remove_device/1"},{"type":"opaque","title":"grisp.device/0","doc":"An object representing the running device.","ref":"grisp.html#t:device/0"},{"type":"type","title":"grisp.slot/0","doc":"Designates a hardware slot on the GRiSP board. This should be the lower case\nversion of the slot names printed on the board itself.\n\nE.g. `spi1` for _SPI1_ or `gpio2` for _GPIO2_.","ref":"grisp.html#t:slot/0"},{"type":"module","title":"grisp_barebox","doc":null,"ref":"grisp_barebox.html"},{"type":"function","title":"grisp_barebox.commit/0","doc":null,"ref":"grisp_barebox.html#commit/0"},{"type":"function","title":"grisp_barebox.get/1","doc":null,"ref":"grisp_barebox.html#get/1"},{"type":"function","title":"grisp_barebox.get_all/0","doc":null,"ref":"grisp_barebox.html#get_all/0"},{"type":"function","title":"grisp_barebox.load/0","doc":null,"ref":"grisp_barebox.html#load/0"},{"type":"function","title":"grisp_barebox.set/2","doc":null,"ref":"grisp_barebox.html#set/2"},{"type":"module","title":"grisp_bitmap","doc":"Bitstring editing and printing.","ref":"grisp_bitmap.html"},{"type":"function","title":"grisp_bitmap.get_bits/3","doc":"Get a part of a bitstring.","ref":"grisp_bitmap.html#get_bits/3"},{"type":"function","title":"Example - grisp_bitmap.get_bits/3","doc":"```\n 1> grisp_bitmap:get_bits(<<1,2,3,4,5>>, 6, 2).\n <<1:2>>\n```","ref":"grisp_bitmap.html#get_bits/3-example"},{"type":"function","title":"grisp_bitmap.get_bytes/3","doc":"","ref":"grisp_bitmap.html#get_bytes/3"},{"type":"function","title":"grisp_bitmap.pp/1","doc":"Print binary as hexadecimal numbers.","ref":"grisp_bitmap.html#pp/1"},{"type":"function","title":"Example - grisp_bitmap.pp/1","doc":"```\n 2> grisp_bitmap:pp(<<16#f2, 17>>).\n F2 11\n ok\n```","ref":"grisp_bitmap.html#pp/1-example"},{"type":"function","title":"grisp_bitmap.pp/2","doc":"Print binary as numbers.","ref":"grisp_bitmap.html#pp/2"},{"type":"function","title":"Example - grisp_bitmap.pp/2","doc":"```\n 3> grisp_bitmap:pp(<<16#f2, 17>>, nib).\n 1111 0010  0001 0001\n ok\n 4> grisp_bitmap:pp(<<16#f2, 17>>, #{display => bin}).\n 1111 0010  0001 0001\n ok\n```","ref":"grisp_bitmap.html#pp/2-example"},{"type":"function","title":"grisp_bitmap.set_bits/3","doc":"Replace a part of a bitsting.","ref":"grisp_bitmap.html#set_bits/3"},{"type":"function","title":"Example - grisp_bitmap.set_bits/3","doc":"```\n5> grisp_bitmap:pp(grisp_bitmap:set_bits(<<2#00000000>>, 6, <<2#1:1>>), bin).\n00000010\nok\n```","ref":"grisp_bitmap.html#set_bits/3-example"},{"type":"function","title":"grisp_bitmap.set_bytes/3","doc":"","ref":"grisp_bitmap.html#set_bytes/3"},{"type":"type","title":"grisp_bitmap.coding/0","doc":"","ref":"grisp_bitmap.html#t:coding/0"},{"type":"module","title":"grisp_eeprom","doc":"","ref":"grisp_eeprom.html"},{"type":"function","title":"grisp_eeprom.read/3","doc":"","ref":"grisp_eeprom.html#read/3"},{"type":"function","title":"grisp_eeprom.write/3","doc":"","ref":"grisp_eeprom.html#write/3"},{"type":"module","title":"grisp_gpio","doc":"GRiSP General Purpose Input/Output (GPIO) API.\n\nGeneral Purpose Input / Output (GPIO) is used to control digital signals on a\npin. The digital values `0` and `1` correspond to a low or high voltage\nrespectively. On GRiSP the voltage for GPIO pins is 3.3V.\n\nA pin can be controlled either as an output pin or an input pin. For an output\npin, it is possible to set or get the current value. For an input pin, it is\npossible to get the current value.","ref":"grisp_gpio.html"},{"type":"module","title":"Pin Mappings - grisp_gpio","doc":"For PMOD connectors, the number column (*#*) maps to the respective PMOD\npin (see [Figure 1](#figure_1)).\n\n#### GRiSP 2 Pin Mappings\n\n| ID          | Slot     | Type    | #  | Schematic | Direction | Description              |\n| ----------- | -------- | ------- | -- | --------- | --------- | ------------------------ |\n| `gpio1_1`   | GPIO1    | PMOD 1A | 1  | X1404.1   | In/Out    |                          |\n| `gpio1_2`   | GPIO1    | PMOD 1A | 2  | X1404.2   | In/Out    |                          |\n| `gpio1_3`   | GPIO1    | PMOD 1A | 3  | X1404.3   | In/Out    |                          |\n| `gpio1_4`   | GPIO1    | PMOD 1A | 4  | X1404.4   | In/Out    |                          |\n| `gpio1_7`   | GPIO1    | PMOD 1A | 7  | X1404.7   | In/Out    |                          |\n| `gpio1_8`   | GPIO1    | PMOD 1A | 8  | X1404.8   | In/Out    |                          |\n| `gpio1_9`   | GPIO1    | PMOD 1A | 9  | X1404.9   | In/Out    |                          |\n| `gpio1_10`  | GPIO1    | PMOD 1A | 10 | X1404.10  | In/Out    |                          |\n| `gpio_1_3`  | GPIO_1   | Generic |    | X1300.3   | In/Out    |                          |\n| `gpio_1_4`  | GPIO_1   | Generic |    | X1300.4   | In/Out    |                          |\n| `gpio_1_5`  | GPIO_1   | Generic |    | X1300.5   | In/Out    |                          |\n| `gpio_1_6`  | GPIO_1   | Generic |    | X1300.6   | In/Out    |                          |\n| `gpio_2_3`  | GPIO_2/4 | Generic |    | X1301.3   | In/Out    |                          |\n| `gpio_2_4`  | GPIO_2/4 | Generic |    | X1301.4   | In/Out    |                          |\n| `gpio_2_5`  | GPIO_2/4 | Generic |    | X1301.5   | In/Out    |                          |\n| `gpio_2_6`  | GPIO_2/4 | Generic |    | X1301.6   | In/Out    |                          |\n| `gpio_2_7`  | GPIO_2/4 | Generic |    | X1301.7   | In/Out    |                          |\n| `gpio_2_8`  | GPIO_2/4 | Generic |    | X1301.8   | In/Out    |                          |\n| `gpio_2_9`  | GPIO_2/4 | Generic |    | X1301.9   | In/Out    |                          |\n| `gpio_2_10` | GPIO_2/4 | Generic |    | X1301.10  | In/Out    |                          |\n| `gpio_2_11` | GPIO_2/4 | Generic |    | X1301.11  | In/Out    |                          |\n| `gpio_2_12` | GPIO_2/4 | Generic |    | X1301.12  | In/Out    |                          |\n| `gpio_2_13` | GPIO_2/4 | Generic |    | X1301.13  | In/Out    |                          |\n| `gpio_2_14` | GPIO_2/4 | Generic |    | X1301.14  | In/Out    |                          |\n| `led1_r`    | LED 1    | LED     |    | RGB1red   | Out       | _Reserved by LED driver_ |\n| `led1_g`    | LED 1    | LED     |    | RGB1green | Out       | _Reserved by LED driver_ |\n| `led1_b`    | LED 1    | LED     |    | RGB1blue  | Out       | _Reserved by LED driver_ |\n| `led2_r`    | LED 2    | LED     |    | RGB2red   | Out       | _Reserved by LED driver_ |\n| `led2_g`    | LED 2    | LED     |    | RGB2green | Out       | _Reserved by LED driver_ |\n| `led2_b`    | LED 2    | LED     |    | RGB3blue  | Out       | _Reserved by LED driver_ |\n| `jumper_1`  | Mode     | Jumper  |    | JUMPER1   | In        | Mode Switch Jumper State |\n| `jumper_2`  | Mode     | Jumper  |    | JUMPER2   | In        | Mode Switch Jumper State |\n| `jumper_3`  | Mode     | Jumper  |    | JUMPER3   | In        | Mode Switch Jumper State |\n| `jumper_4`  | Mode     | Jumper  |    | JUMPER4   | In        | Mode Switch Jumper State |\n| `jumper_5`  | Mode     | Jumper  |    | JUMPER5   | In        | Mode Switch Jumper State |\n\n> #### Warning {: .warning}\n> Reserved pins should only be carefully used if their drivers are not in use\n\n#### GRiSP 1 Pin Mappings\n\n| ID          | Slot     | Type    | #  | Schematic | Direction | Description              |\n| ----------- | -------- | ------- | -- | --------- | --------- | ------------------------ |\n| `gpio1_1`   | GPIO1    | PMOD 1  | 1  | X502.1    | In/Out    |                          |\n| `gpio1_2`   | GPIO1    | PMOD 1  | 2  | X502.2    | In/Out    |                          |\n| `gpio1_3`   | GPIO1    | PMOD 1  | 3  | X502.3    | In/Out    |                          |\n| `gpio1_4`   | GPIO1    | PMOD 1  | 4  | X502.4    | In/Out    |                          |\n| `gpio2_1`   | GPIO2    | PMOD 1  | 1  | X503.1    | In/Out    |                          |\n| `gpio2_2`   | GPIO2    | PMOD 1  | 2  | X503.2    | In/Out    |                          |\n| `gpio2_3`   | GPIO2    | PMOD 1  | 3  | X503.3    | In/Out    |                          |\n| `gpio2_4`   | GPIO2    | PMOD 1  | 4  | X503.4    | In/Out    |                          |\n| `led1_r`    | LED 1    | LED     |    | RGB1red   | Out       | _Reserved by LED driver_ |\n| `led1_g`    | LED 1    | LED     |    | RGB1green | Out       | _Reserved by LED driver_ |\n| `led1_b`    | LED 1    | LED     |    | RGB1blue  | Out       | _Reserved by LED driver_ |\n| `led2_r`    | LED 2    | LED     |    | RGB2red   | Out       | _Reserved by LED driver_ |\n| `led2_g`    | LED 2    | LED     |    | RGB2green | Out       | _Reserved by LED driver_ |\n| `led2_b`    | LED 2    | LED     |    | RGB3blue  | Out       | _Reserved by LED driver_ |\n| `jumper_1`  | Mode     | Jumper  |    | JUMPER1   | In        | Mode Switch Jumper State |\n| `jumper_2`  | Mode     | Jumper  |    | JUMPER2   | In        | Mode Switch Jumper State |\n| `jumper_3`  | Mode     | Jumper  |    | JUMPER3   | In        | Mode Switch Jumper State |\n| `jumper_4`  | Mode     | Jumper  |    | JUMPER4   | In        | Mode Switch Jumper State |\n| `jumper_5`  | Mode     | Jumper  |    | JUMPER5   | In        | Mode Switch Jumper State |\n| `spi1_pin7` | SPI1     | PMOD 2A | 7  | X501.7    | In/Out    |                          |\n| `spi1_pin8` | SPI1     | PMOD 2A | 8  | X501.8    | In/Out    |                          |\n| `spi1_pin9` | SPI1     | PMOD 2A | 9  | X501.9    | In/Out    | _Reserved by SPI driver_ |\n| `spi1_pin10`| SPI1     | PMOD 2A | 10 | X501.10   | In/Out    | _Reserved by SPI driver_ |\n| `spi1_pin1` | SPI1     | PMOD 2A | 1  | X501.1    | In/Out    | _Reserved by SPI driver_ |\n| `spi2_pin1` | SPI2     | PMOD 2  | 1  | X509.1    | In/Out    | _Reserved by SPI driver_ |\n| `uart_1_cts`| UART     | PMOD 3  | 1  | X508.1    | In/Out    |                          |\n| `uart_2_txd`| UART     | PMOD 3  | 2  | X508.2    | In/Out    |                          |\n| `uart_3_rxd`| UART     | PMOD 3  | 3  | X508.3    | In/Out    |                          |\n| `uart_4_rts`| UART     | PMOD 3  | 4  | X508.4    | In/Out    |                          |\n\n> #### Warning {: .warning}\n> Reserved pins should only be carefully used if their drivers are not in use","ref":"grisp_gpio.html#module-pin-mappings"},{"type":"module","title":"PMOD Pin Numbers - grisp_gpio","doc":"Figure 1. PMOD connectors as seen from the side of a GRiSP board with number mappings\n\nPMOD Type A consists of:\n * 4 × data, pins #1-4 \n * 1 × ground, pin #5 \n * 1 × 3.3V power, pin #6 \n\nPMOD Type B consists of:\n * 8 × data, pins #1-4 and #7-10 \n * 1 × ground, pins #5 and #11 \n * 1 × 3.3V power, pins #6 and #12","ref":"grisp_gpio.html#module-pmod-pin-numbers"},{"type":"function","title":"grisp_gpio.get/1","doc":"Returns the current value of a pin.\n\nReturns the actual value for input pins or the currently set value for output\npins.","ref":"grisp_gpio.html#get/1"},{"type":"function","title":"Examples - grisp_gpio.get/1","doc":"To see whether the red component of LED 1 is enabled:\n```\n1> LED1R = grisp_gpio:open(led1_r, #{mode => {output, 0}}).\n#Ref<0.2691682867.116916226.176944>\n2> grisp_gpio:get(LED1R).\n0\n3> grisp_gpio:set(LED1R, 1).\nok\n2> grisp_gpio:get(LED1R).\n1\n```\n\nTo see whether Mode Jumper 1 is on or off:\n```\n1> Jumper1 = grisp_gpio:open(jumper_1, #{mode => input}).\n#Ref<0.2691682867.116916226.176944>\n2> grisp_gpio:get(Jumper1).\n0\n```\nFlip the jumper\n```\n3> grisp_gpio:get(Jumper1).\n1\n```","ref":"grisp_gpio.html#get/1-examples"},{"type":"function","title":"grisp_gpio.open/1","doc":"","ref":"grisp_gpio.html#open/1"},{"type":"function","title":"grisp_gpio.open/2","doc":"Creates a reference to a GPIO pin.\nIf no mode is given in the options, it defaults to `{output, 0}`.\n\n<!-- tabs-open -->","ref":"grisp_gpio.html#open/2"},{"type":"function","title":"LED Output Example - grisp_gpio.open/2","doc":"Open the GPIO pin of the red component of LED 1 as an output pin with initial\nvalue of `0`:\n```\n1> grisp_gpio:open(led1_r, #{mode => {output, 0}}).\n#Ref<0.2691682867.116916226.176944>\n```","ref":"grisp_gpio.html#open/2-led-output-example"},{"type":"function","title":"Jumper Input Example - grisp_gpio.open/2","doc":"Open the GPIO pin of Mode Jumper 1 as an input pin:\n```\n1> grisp_gpio:open(jumper_1, #{mode => input}).\n#Ref<0.2691682867.116916226.176944>\n```\n<!-- tabs-close -->","ref":"grisp_gpio.html#open/2-jumper-input-example"},{"type":"function","title":"grisp_gpio.set/2","doc":"Sets the current value of an output pin.","ref":"grisp_gpio.html#set/2"},{"type":"function","title":"Example - grisp_gpio.set/2","doc":"Turn off the red component of LED 1:\n```\n1> LED1R = grisp_gpio:open(led1_r, #{mode => {output, 0}}).\n#Ref<0.2691682867.116916226.176944>\n2> grisp_gpio:set(LED1R, 0).\nok\n```\nTurn on the red component of LED 1:\n```\n3> grisp_gpio:set(LED1R, 1).\nok\n```","ref":"grisp_gpio.html#set/2-example"},{"type":"type","title":"grisp_gpio.mode/0","doc":"","ref":"grisp_gpio.html#t:mode/0"},{"type":"type","title":"grisp_gpio.opts/0","doc":"","ref":"grisp_gpio.html#t:opts/0"},{"type":"type","title":"grisp_gpio.pin/0","doc":"","ref":"grisp_gpio.html#t:pin/0"},{"type":"opaque","title":"grisp_gpio.ref/0","doc":"","ref":"grisp_gpio.html#t:ref/0"},{"type":"type","title":"grisp_gpio.value/0","doc":"","ref":"grisp_gpio.html#t:value/0"},{"type":"module","title":"grisp_hw","doc":"GRiSP hardware access.","ref":"grisp_hw.html"},{"type":"function","title":"grisp_hw.eeprom_read/0","doc":"Reads GRiSP meta data from EEPROM","ref":"grisp_hw.html#eeprom_read/0"},{"type":"function","title":"Example - grisp_hw.eeprom_read/0","doc":"```\n1> grisp_hw:eeprom_read().\n{ok, #{grisp_batch => 1,\n       grisp_pcb_variant => 1,\n       grisp_pcb_version => \"1.2\",\n       grisp_prod_date => {{2021,8,27},{0,0,0}},\n       grisp_serial => 1002,\n       grisp_version => \"2\"}}\n```","ref":"grisp_hw.html#eeprom_read/0-example"},{"type":"function","title":"grisp_hw.eeprom_reset_crc/0","doc":"Fixes CRC bytes for pre-production boards","ref":"grisp_hw.html#eeprom_reset_crc/0"},{"type":"function","title":"grisp_hw.platform/0","doc":"Returns the platform name as an atom.","ref":"grisp_hw.html#platform/0"},{"type":"module","title":"grisp_i2c","doc":"GRiSP I²C API.\n\n[Inter-Integrated Circuit (I²C)](https://en.wikipedia.org/wiki/I²C) is a\nsynchronous addressable serial communication bus. Two types of\nnodes can exist, controllers and targets. Multiple controllers and targets\ncan exist on the bus simultaneously. Controllers generate the clock signal\nand initiate communication with targets. Targets follow the clock and respond\nwhen addressed. A GRiSP board acts as a controller to any connected target\nperipherals on the I²C buses.\n\nIn GRiSP there are two main I²C buses, one \"internal\" bus (`i2c0`) and\none \"external\" bus (`i2c1`). The internal bus talks to targets on the SoM and\nthe board itself, such as EEPROM memory or the 1-Wire controller. The\nexternal bus talks to any connected target peripherals, either on the PMOD\nconnector or the raised I²C pins.\n\nCommunication consists of either read or write messages. A read or write must\nbe addressed to a specific target (chip). A target is addressed by a 7-bit\naddress. The registry to use is specified in a target specific way. E.g. to\nread a register, some targets expect a write message with the register address\nas a value, followed by a read message.\n\n\n> #### Note {: .tip}\n> Please refer to the specification for a specific target chip for\ninstructions of what messages to send and receive, and how the registers are\nstructured.","ref":"grisp_i2c.html"},{"type":"function","title":"grisp_i2c.buses/0","doc":"Lists I²C buses.","ref":"grisp_i2c.html#buses/0"},{"type":"function","title":"Example - grisp_i2c.buses/0","doc":"```\n1> grisp_i2c:buses()\n#{i2c0 => #{name => i2c0,path => <<\"/dev/i2c-0\">>},\n  i2c1 => #{name => i2c1,path => <<\"/dev/i2c-1\">>}}\n```","ref":"grisp_i2c.html#buses/0-example"},{"type":"function","title":"grisp_i2c.detect/1","doc":"Detects I²C devices on a bus.\n\nReturns the address of each found device.","ref":"grisp_i2c.html#detect/1"},{"type":"function","title":"Example - grisp_i2c.detect/1","doc":"```\n3> [io_lib:format(\"0x~.16B\", [Target]) || Target <- grisp_i2c:detect(I2C0)].\n[\"0x18\",\"0x36\",\"0x37\",\"0x52\",\"0x57\",\"0x5A\",\"0x5F\"]\n```","ref":"grisp_i2c.html#detect/1-example"},{"type":"function","title":"grisp_i2c.open/1","doc":"Opens an I²C bus device by name.","ref":"grisp_i2c.html#open/1"},{"type":"function","title":"Example - grisp_i2c.open/1","doc":"```\n2> I2C0 = grisp_i2c:open(i2c1).\n#Ref<0.4157010815.3886678017.238942>\n```","ref":"grisp_i2c.html#open/1-example"},{"type":"function","title":"grisp_i2c.read/4","doc":"Performs a simplified read from a register on an I²C chip.\n\nThis function sends a write message with the single byte register address as\nvalue, then a read message of the specified length.\n\n> #### Note {: .tip}\n> Only some chips support this simple operation. Please consult\nthe target chip specification.","ref":"grisp_i2c.html#read/4"},{"type":"function","title":"Example - grisp_i2c.read/4","doc":"```\n4> grisp_i2c:read(I2C1, TargetAddr, RegAddr, 1).\n<<255>>\n```","ref":"grisp_i2c.html#read/4-example"},{"type":"function","title":"grisp_i2c.transfer/2","doc":"Transfers I²C messages on a bus.","ref":"grisp_i2c.html#transfer/2"},{"type":"function","title":"Examples - grisp_i2c.transfer/2","doc":"```\n6> grisp_i2c:transfer(I2C1, [{read, TargetAddr, RegAddr, 1}])\n[<<255>>]\n7> grisp_i2c:transfer(I2C1, [{write, TargetAddr, RegAddr, < >}])\nok\n```","ref":"grisp_i2c.html#transfer/2-examples"},{"type":"function","title":"grisp_i2c.write/4","doc":"Performs a simplified write to a register on an I²C chip.\n\nThis function sends a write message with the single byte register address as\nthe first byte and then the data as the following bytes.\n\n> #### Note {: .tip}\n> Only some chips support this simple operation. Please consult\nthe target chip specification.","ref":"grisp_i2c.html#write/4"},{"type":"function","title":"Example - grisp_i2c.write/4","doc":"```\n5> grisp_i2c:write(I2C1, TargetAddr, RegAddr, < >).\nok\n```","ref":"grisp_i2c.html#write/4-example"},{"type":"opaque","title":"grisp_i2c.bus/0","doc":"","ref":"grisp_i2c.html#t:bus/0"},{"type":"type","title":"grisp_i2c.bus_name/0","doc":"","ref":"grisp_i2c.html#t:bus_name/0"},{"type":"type","title":"grisp_i2c.bus_path/0","doc":"","ref":"grisp_i2c.html#t:bus_path/0"},{"type":"type","title":"grisp_i2c.error/0","doc":"","ref":"grisp_i2c.html#t:error/0"},{"type":"type","title":"grisp_i2c.length/0","doc":"","ref":"grisp_i2c.html#t:length/0"},{"type":"type","title":"grisp_i2c.message/0","doc":"","ref":"grisp_i2c.html#t:message/0"},{"type":"type","title":"grisp_i2c.reg_addr/0","doc":"","ref":"grisp_i2c.html#t:reg_addr/0"},{"type":"type","title":"grisp_i2c.target_addr/0","doc":"","ref":"grisp_i2c.html#t:target_addr/0"},{"type":"module","title":"grisp_info","doc":"GRiSP general information.\n\nWhen running grisp on a host machine during development, the functions return:\n\n```\n1> grisp_info:hardware().\n#{\n    platform => host,\n    version => <<\"1\">>,\n    serial => <<\"0000\">>\n    batch => 1,\n    pcb_variant => 1\n    pcb_version => <<\"1\">>,\n    prod_date => {{2022, 1, 28},{0,0,0}}\n}\n2> grisp_info:boot().\n#{\n    boot => host,\n    valid => host,\n    next => host\n}\n```\n\nFor grisp_info:software() to be defined during local development, a term file\nnamed `MANIFEST` must exists in the current directory and looks like:\n\n```\n%% coding: utf-8\n{platform, <<\"grisp2\">>}.\n{id, <<\"83613140847d62b47c3f220b6f43e2de561be441\">>}.\n{relname, <<\"my_app\">>}.\n{relvsn, <<\"0.1.0\">>}.\n{profiles, [dev]}.\n{package, [\n    {toolchain, [{revision, <<\"516707805fd33285679e9e64585b995d31d926ec\">>}]},\n    {rtems, [{version, <<\"5\">>}]},\n    {otp, [{version, <<\"26.2.5.4\">>}]}\n]}.\n```","ref":"grisp_info.html"},{"type":"function","title":"grisp_info.boot/0","doc":"","ref":"grisp_info.html#boot/0"},{"type":"function","title":"grisp_info.hardware/0","doc":"","ref":"grisp_info.html#hardware/0"},{"type":"function","title":"grisp_info.software/0","doc":"","ref":"grisp_info.html#software/0"},{"type":"function","title":"grisp_info.software/1","doc":"","ref":"grisp_info.html#software/1"},{"type":"type","title":"grisp_info.boot_info/0","doc":"","ref":"grisp_info.html#t:boot_info/0"},{"type":"type","title":"grisp_info.boot_source/0","doc":"","ref":"grisp_info.html#t:boot_source/0"},{"type":"type","title":"grisp_info.boot_system/0","doc":"","ref":"grisp_info.html#t:boot_system/0"},{"type":"type","title":"grisp_info.hardware_info/0","doc":"","ref":"grisp_info.html#t:hardware_info/0"},{"type":"type","title":"grisp_info.software_info/0","doc":"","ref":"grisp_info.html#t:software_info/0"},{"type":"module","title":"grisp_led","doc":"","ref":"grisp_led.html"},{"type":"function","title":"grisp_led.color/2","doc":"Set the color of an LED.","ref":"grisp_led.html#color/2"},{"type":"function","title":"Examples - grisp_led.color/2","doc":"```\n1> grisp_led:color(1, red).\nok\n2> grisp_led:color(2, {0, 1, 0}).\nok\n```","ref":"grisp_led.html#color/2-examples"},{"type":"function","title":"grisp_led.flash/3","doc":"Flash an LED in an on/off pattern with the specified color.","ref":"grisp_led.html#flash/3"},{"type":"function","title":"Examples - grisp_led.flash/3","doc":"```\n1> grisp_led:flash(2, blue, 500).\nok\n```","ref":"grisp_led.html#flash/3-examples"},{"type":"function","title":"grisp_led.off/1","doc":"Turn off an LED.","ref":"grisp_led.html#off/1"},{"type":"function","title":"grisp_led.pattern/2","doc":"Animate an LED with a pattern of colors and intervals.","ref":"grisp_led.html#pattern/2"},{"type":"function","title":"Examples - grisp_led.pattern/2","doc":"```\n1> grisp_led:pattern(1, [{300, green}, {500, yellow}, {700, red}, {infinity, off}]).\nok\n2> Rainbow = [{300, {R, G, B}} || R <- [0,1], G <- [0,1], B <- [0,1], {R, G, B} =/= {0, 0, 0}].\n[{300,{0,0,1}},\n {300,{0,1,0}},\n {300,{0,1,1}},\n {300,{1,0,0}},\n {300,{1,0,1}},\n {300,{1,1,0}},\n {300,{1,1,1}}]\n3> grisp_led:pattern(2, Rainbow).\nok\n```\n\nThe color can also be specified using functions as generators\ninstead of explicitly stating the color :\n\n```\n2> Random = fun() -> {rand:uniform(2) - 1, rand:uniform(2) -1, rand:uniform(2) - 1} end.\n#Fun \n3> grisp_led:pattern(1, [{100, Random}]).\n```\n\nAs well as by composing lists of intervals and pattern functions :\n\n```\n4> Funs = [ fun() -> {X rem 2, rand:uniform(2) - 1 , 1} end || X <- lists:seq(1,10) ].\n[#Fun , ...\n5> Intervals = lists:seq(1000,1900,100).\n[1000,1100,1200,1300,1400,1500,1600,1700,1800,1900]\n6> Result = lists:zip(Intervals, Funs).\n[{1000,#Fun },...\n7> grisp_led:pattern(1, Result).\n```","ref":"grisp_led.html#pattern/2-examples"},{"type":"function","title":"grisp_led.read/1","doc":"","ref":"grisp_led.html#read/1"},{"type":"type","title":"grisp_led.color/0","doc":"","ref":"grisp_led.html#t:color/0"},{"type":"type","title":"grisp_led.color_name/0","doc":"","ref":"grisp_led.html#t:color_name/0"},{"type":"type","title":"grisp_led.color_value/0","doc":"","ref":"grisp_led.html#t:color_value/0"},{"type":"type","title":"grisp_led.pattern/0","doc":"","ref":"grisp_led.html#t:pattern/0"},{"type":"type","title":"grisp_led.position/0","doc":"","ref":"grisp_led.html#t:position/0"},{"type":"type","title":"grisp_led.time/0","doc":"","ref":"grisp_led.html#t:time/0"},{"type":"module","title":"grisp_nmea","doc":null,"ref":"grisp_nmea.html"},{"type":"function","title":"grisp_nmea.parse/1","doc":null,"ref":"grisp_nmea.html#parse/1"},{"type":"type","title":"grisp_nmea.message_id/0","doc":null,"ref":"grisp_nmea.html#t:message_id/0"},{"type":"type","title":"grisp_nmea.talker_id/0","doc":null,"ref":"grisp_nmea.html#t:talker_id/0"},{"type":"module","title":"grisp_onewire","doc":"Driver API for the\n[DS2482-100 Single-Channel 1-Wire Master](https://datasheets.maximintegrated.com/en/ds/DS2482-100.pdf)\n\nThe functions in this module refer to the function commands documented in the\nmasters [data sheet](https://datasheets.maximintegrated.com/en/ds/DS2482-100.pdf).\nEach function has a hexadecimal command code that is referenced in the\nspecification sheet.","ref":"grisp_onewire.html"},{"type":"function","title":"grisp_onewire.bus_reset/0","doc":"Reset the bus and check the register for devices.\n\nThis function can only be used inside of a `transaction/1` call.","ref":"grisp_onewire.html#bus_reset/0"},{"type":"function","title":"Return Value Description - grisp_onewire.bus_reset/0","doc":"| Atom                | Description                                                 |\n| ------------------- | ----------------------------------------------------------- |\n| `nothing_present`   | No device on the bus detected                               |\n| `presence_detected` | Some devices on the bus detected                            |\n| `short_detected`    | A short circuit between data and ground on the bus detected |\n\n_Command code: `b4h`._","ref":"grisp_onewire.html#bus_reset/0-return-value-description"},{"type":"function","title":"grisp_onewire.detect/0","doc":"Reset device and activate active pullup (APU).\n\nThis function can only be used inside of a `transaction/1` call.\n\nSee also: `reset/0`, `write_config/1`.","ref":"grisp_onewire.html#detect/0"},{"type":"function","title":"grisp_onewire.read_byte/0","doc":"Read one data byte from the 1-Wire line.\n\nThis function can only be used inside of a `transaction/1` call.\n\n_Command codes: `96h`, `e1h`_","ref":"grisp_onewire.html#read_byte/0"},{"type":"function","title":"grisp_onewire.reset/0","doc":"Reset the 1-Wire Master and terminate any ongoing 1-Wire communication.\n\nThis function can only be used inside of a `transaction/1` call.\n\n_Command code: `f0h`._","ref":"grisp_onewire.html#reset/0"},{"type":"function","title":"grisp_onewire.search/0","doc":"Search the bus for devices.\n\nThis function can only be used inside of a `transaction/1` call.\n\nIf there are connected devices, i.e., `bus_reset/0` returns\n`presence_detected`, this function provides a list of the unique 64-bit\naddresses of all detected devices.\nOtherwise, the return values match the values from `bus_reset/0` or\n`fail` for other failures during the search.\nThe addresses are represented as lists of eight byte values.","ref":"grisp_onewire.html#search/0"},{"type":"function","title":"Example - grisp_onewire.search/0","doc":"With five DS18B20 temperature sensors connected one can list the five device\nIDs:\n```\n 1> grisp_onewire:transaction(fun grisp_onewire:search/0).\n [[40,255,203,173,80,23,4,182],\n [40,255,67,77,96,23,5,138],\n [40,255,190,25,96,23,3,203],\n [40,255,54,42,96,23,5,35],\n [40,255,18,91,96,23,3,62]]\n```","ref":"grisp_onewire.html#search/0-example"},{"type":"function","title":"grisp_onewire.transaction/1","doc":"Run a 1-Wire transaction.\n\nUse this function to make sure that there is only one process running on the\n1-Wire.","ref":"grisp_onewire.html#transaction/1"},{"type":"function","title":"Example - grisp_onewire.transaction/1","doc":"```\n 2> grisp_onewire:transaction(fun() ->\n                                 presence_detected = grisp_onewire:bus_reset(),\n                                 grisp_onewire:write_byte(16#cc)\n                              end).\n ok\n```","ref":"grisp_onewire.html#transaction/1-example"},{"type":"function","title":"grisp_onewire.write_byte/1","doc":"Write one data byte to the 1-Wire line.\n\nThis function can only be used inside of a `transaction/1` call.\n\n_Command code: `a5h`__","ref":"grisp_onewire.html#write_byte/1"},{"type":"function","title":"grisp_onewire.write_config/1","doc":"Write configuration into 1-Wire master register.\n\nThis function can only be used inside of a `transaction/1` call.\n\nThe default configuration is `0`, i.e., all three configurable bits set to\n`0`. This corresponds to an empty list. Each atom in the list activates the\ncorresponding configuration (sets the bit to `1`) and each atom not present in\nthe list leads to a deactivation (sets the bit to `0`).","ref":"grisp_onewire.html#write_config/1"},{"type":"function","title":"Atom to Integer to Configuration Bit Mapping - grisp_onewire.write_config/1","doc":"| Atom        | Integer | Configuration Bit | Activates              |\n| ----------- | ------- | ----------------- | ---------------------- |\n| `apu`       | `1`     | Bit 0 (APU)       | Active pullup          |\n| `spu`       | `4`     | Bit 2 (SPU)       | Strong pullup          |\n| `overdrive` | `8`     | Bit 3 (1WS)       | 1-Wire overdrive speed |","ref":"grisp_onewire.html#write_config/1-atom-to-integer-to-configuration-bit-mapping"},{"type":"function","title":"Example - grisp_onewire.write_config/1","doc":"To activate active pullup and overdrive speed use:\n```\n 3> grisp_onewire:transaction(fun() -> grisp_onewire:write_config([apu, overdrive]) end).\n ok\n```\n\nThis is the same as:\n```\n 4> grisp_onewire:transaction(fun() -> grisp_onewire:write_config(1 bor 8) end).\n ok\n```\n\n_Command code: `d2h`._","ref":"grisp_onewire.html#write_config/1-example"},{"type":"module","title":"grisp_pwm","doc":"GriSP Pulse Width Modulation (PWM) API.\n\nPulse Width Modulation (PWM) is used to generate a rectangular wave with a varying duty cycle\nto control the average power or amplitude delivered.\nThe ARM Cortex-A7 has eight PWM units that can be multiplexed to drive a few different pins.\n```\n  1> grisp_pwm:start_driver().\n  {device,pwm,grisp_pwm,<0.353.0>,\n        #Ref<0.838610995.3080454145.146844>}\n  2> grisp_pwm:open(gpio1_8, default, 0.75).\n  ok\n```\n\nThis creates a rectangular wave on pin `gpio1_8` with a 155.5μs cycle time (6.43MHz) and a duty cycle of 75%\n(see [Figure 1](#figure_1)).\n\nYou can change the duty cycle by setting a new sample:\n```\n3> grisp_pwm:set_sample(gpio1_8, 0.5).\nok\n```\n\nIf you want to stop using PWM on this pin you can call:\n```\n 4> grisp_pwm:close(gpio1_8).\n ok\n```\n\n \n![image](assets/pwm_example.png)\nFigure 1. Oscilloscope trace with a 0.75 % duty cycle and the default configuration.\n\n<!-- tabs-open -->","ref":"grisp_pwm.html"},{"type":"module","title":"Ramp Up Example - grisp_pwm","doc":"Youn can ramp up the duty cycle from 0% to 100% in one second like this:\n```\n1> RampSample = fun(X) -> grisp_pwm:set_sample(gpio1_8, (X/100)), timer:sleep(10) end.\n#Fun \n2> [RampSample(X) || X <- lists:seq(1, 100)].\n[ok,ok,ok,ok,ok,ok,ok,ok,ok,ok,ok,ok,ok,ok,ok,ok,ok,ok,ok,\nok,ok,ok,ok,ok,ok,ok,ok,ok,ok|...]\n```","ref":"grisp_pwm.html#module-ramp-up-example"},{"type":"module","title":"Sinusoidal Example - grisp_pwm","doc":"You can create a sine wave like this:\n\n```\n1> SinSample = fun(X) -> V = math:sin(math:pi()/2*(X/10))/2+0.5, grisp_pwm:set_sample(gpio1_8, V), timer:sleep(10) end.\n#Fun \n2> [SinSample(X) || X <- lists:seq(1, 1000)].\n[ok,ok,ok,ok,ok,ok,ok,ok,ok,ok,ok,ok,ok,ok,ok,ok,ok,ok,ok,\nok,ok,ok,ok,ok,ok,ok,ok,ok,ok|...]\n```\n<!-- tabs-close -->","ref":"grisp_pwm.html#module-sinusoidal-example"},{"type":"module","title":"Pin Mappings - grisp_pwm","doc":"| ID        | Slot    | Type    | # | Schematic |\n| --------- | ------- | ------- | - | --------- |\n| `gpio1_2` | GPIO1   | PMOD 1A | 2 | X1404.2   |\n| `gpio1_4` | GPIO1   | PMOD 1A | 4 | X1404.4   |\n| `gpio1_8` | GPIO1   | PMOD 1A | 8 | X1404.8   |\n| `gpio_2_6`| GPIO_2/4| Generic |   | X1301.6   |\n| `spi2_7`  | SPI2    | SPI     |   | X1402.7   |\n| `uart_8`  | UART    | UART    |   | X1405.8   |\n| `uart_9`  | UART    | UART    |   | X1405.9   |\n| `jtag_4`  | JTAG    | JTAG    |   | X1503.4   |\n| `jtag_8`  | JTAG    | JTAG    |   | X1503.8   |\n\n> #### Note {: .tip}\n> This driver might use pins that are also used by other drivers (GPIO, SPI, UART). When opening a pin the PWM driver\n> takes control and configures the multixplexing so the pin is wired to the corresponding PWM unit.\n> When closing the pin the driver will restore the pevious multiplexing setting, handing back control.\n>\n> Example: `grisp_gpio` is used to set a pin to high, then `grisp_pwm:open/3` is used to drive the pin.\n> After `grisp_pwm:close/1` is called, the pin is set to high and `grisp_gpio` is again in control.","ref":"grisp_pwm.html#module-pin-mappings"},{"type":"function","title":"grisp_pwm.close/1","doc":"Closes a pin.","ref":"grisp_pwm.html#close/1"},{"type":"function","title":"grisp_pwm.open/3","doc":"Opens a pin and sets a configuration and a sample.\n\n<!-- tabs-open -->","ref":"grisp_pwm.html#open/3"},{"type":"function","title":"Default Config - grisp_pwm.open/3","doc":"The default configuratin uses the `ipg_clk` clock, a prescale of 10 and a period of `<<1024:16>>`.\nThis results in a 155.5μs cycle time (6.43MHz) and 10bit resolution.\n```\n1> grisp_pwm:open(gpio1_8, default, 0.75).\nok\n```","ref":"grisp_pwm.html#open/3-default-config"},{"type":"function","title":"Prescale and Period Config - grisp_pwm.open/3","doc":"Here we set the prescaler to 1 (10 times higher frequency than the default) and the period to `<<512:16>>` (further doubling the frequency and halfing the resolution).\nThe result is a 7,775μs cycle time (128.6MHz) and 9bit resolution.\n```\n1> grisp_pwm:open(gpio1_8, {1, <<512:16>>}, 0.5).\nok\n```","ref":"grisp_pwm.html#open/3-prescale-and-period-config"},{"type":"function","title":"Clock, Prescale and Period Config - grisp_pwm.open/3","doc":"This sets a low frequency clock and results in a 500ms cycle time (2Hz) and 4bit resolution.\n```\n1> grisp_pwm:open(gpio1_8, {ipg_clk_32k, 1, <<16:16>>}, 0.5).\nok\n```\n<!-- tabs-close -->","ref":"grisp_pwm.html#open/3-clock-prescale-and-period-config"},{"type":"function","title":"grisp_pwm.set_sample/2","doc":"Sets a sample to define the duty cycle.\n\nYou can pass a float between 0.0 and 1.0 or a 16bit binary.\nThe binary value must be below or equal to the period (`<<1024:16>>` by default).\n\n<!-- tabs-open -->","ref":"grisp_pwm.html#set_sample/2"},{"type":"function","title":"Float Sample - grisp_pwm.set_sample/2","doc":"This sets the duty cycle to 25%. This method is independent of the period used.\n```\n1> grisp_pwm:set_sample(gpio1_8, 0.25).\nok\n```","ref":"grisp_pwm.html#set_sample/2-float-sample"},{"type":"function","title":"Binary Sample - grisp_pwm.set_sample/2","doc":"This sets the duty cycle to 25% given a period of `<<1024:16>>`.\n```\n1> grisp_pwm:set_sample(gpio1_8, <<256:16>>).\nok\n```\n<!-- tabs-close -->","ref":"grisp_pwm.html#set_sample/2-binary-sample"},{"type":"function","title":"grisp_pwm.start_driver/0","doc":"Starts the driver and registers a PWM device.","ref":"grisp_pwm.html#start_driver/0"},{"type":"type","title":"grisp_pwm.clock/0","doc":"","ref":"grisp_pwm.html#t:clock/0"},{"type":"type","title":"grisp_pwm.config/0","doc":"","ref":"grisp_pwm.html#t:config/0"},{"type":"type","title":"grisp_pwm.period/0","doc":"","ref":"grisp_pwm.html#t:period/0"},{"type":"type","title":"grisp_pwm.pin/0","doc":"","ref":"grisp_pwm.html#t:pin/0"},{"type":"type","title":"grisp_pwm.prescale/0","doc":"","ref":"grisp_pwm.html#t:prescale/0"},{"type":"type","title":"grisp_pwm.sample/0","doc":"","ref":"grisp_pwm.html#t:sample/0"},{"type":"module","title":"grisp_rtems","doc":"GRiSP RTEMS functions.\n\nNIF mappings to RTEMS functions.","ref":"grisp_rtems.html"},{"type":"function","title":"grisp_rtems.clock_get_ticks_per_second/0","doc":"Gets the number of clock ticks per second configured for the application.\n\n*Reference:* [rtems_clock_get_ticks_per_second](https://docs.rtems.org/branches/master/c-user/clock/directives.html#rtems-clock-get-ticks-per-second)","ref":"grisp_rtems.html#clock_get_ticks_per_second/0"},{"type":"function","title":"grisp_rtems.clock_get_ticks_since_boot/0","doc":"Gets the number of clock ticks since some time point during the system\ninitialization or the last overflow of the clock tick counter.\n\n*Reference:* [rtems_clock_get_ticks_since_boot](https://docs.rtems.org/branches/master/c-user/clock/directives.html#rtems-clock-get-ticks-since-boot)","ref":"grisp_rtems.html#clock_get_ticks_since_boot/0"},{"type":"function","title":"grisp_rtems.clock_get_tod/0","doc":"Gets the time of day associated with the current `CLOCK_REALTIME`.\n\n*Reference:* [rtems_clock_get_tod](https://docs.rtems.org/branches/master/c-user/clock/directives.html#rtems-clock-get-tod)","ref":"grisp_rtems.html#clock_get_tod/0"},{"type":"function","title":"grisp_rtems.clock_set/1","doc":"Sets the `CLOCK_REALTIME` to the time of day.\n\n*Reference:* [rtems_clock_set](https://docs.rtems.org/branches/master/c-user/clock/directives.html#rtems-clock-set)","ref":"grisp_rtems.html#clock_set/1"},{"type":"function","title":"grisp_rtems.pwrite/3","doc":"Perform a raw `pwrite` syscall to a device.\n\nThe function is approximately equivalent to the C code:\n\n```\nfd = open(device_path, O_RDWR);\npwrite(fd, buffer.data, buffer.size, offset);\n```\n\n*Reference:* [pwrite](https://linux.die.net/man/2/pwrite)","ref":"grisp_rtems.html#pwrite/3"},{"type":"function","title":"grisp_rtems.unmount/1","doc":"Unmounts the file system instance at the specified mount path.\n\n*Reference:* [rtems_unmount](https://docs.rtems.org/doxygen/branches/master/group__FileSystemTypesAndMount.html#ga4c8f87fc991f94992e0da1f87243f9e0)","ref":"grisp_rtems.html#unmount/1"},{"type":"type","title":"grisp_rtems.time_of_day/0","doc":"","ref":"grisp_rtems.html#t:time_of_day/0"},{"type":"module","title":"grisp_spi","doc":"GRiSP SPI API.\n\n[Serial Peripheral Interface (SPI)](https://en.wikipedia.org/wiki/Serial_Peripheral_Interface)\nis a synchronous serial communication\nprotocol, where a single controller device can control many responder\ndevices. With this API, the GRiSP board acts as a controller and any connected\ndevice is a responder.\n\nSPI has four lines: clock, MOSI, MISO and chip select. MOSI is the data line\nfrom the controller device to the responder devices. MISO is the data line\nfrom the responder devices to the controller device. The controller device is\nregulating the communication speed and synchronization by controlling the\nchip select and clock lines.","ref":"grisp_spi.html"},{"type":"module","title":"Chip Select - grisp_spi","doc":"By default, if the bus aliases `spi1` or `spi2` is used with `open/1`\nthe chip select pin is automatically configured to the default SPI slot chip\nselect pin. When using `open/2` the chip select pin can be any GPIO pin\n(`m:grisp_gpio`).","ref":"grisp_spi.html#module-chip-select"},{"type":"module","title":"Clock - grisp_spi","doc":"The clock line in SPI is a digital I/O line that pulses at a certain\nfrequency. The default SPI clock frequency in GRiSP is 0.1 MHz.\n\nThe clock line polarity and phase should be configured by setting the\nmandatory `clock` mode option (`t:clock/0`) for each message. A polarity\nof `low` means the clock line is idling at `0` and a polarity of `high` means\nthe clock line is idling at `1`. The phase denotes at which edge of the clock\npulse the actual protocol values should be written and read. A phase of\n`leading` means that the controller and responder devices should read or\nwrite values as the pulse starts. A phase of `trailing` means the devices\nshould read or write values as the current pulse ends.\n\nThe settings for the clock line is specific per device and needs to be\nconsulted from the official specification of the device.","ref":"grisp_spi.html#module-clock"},{"type":"module","title":"Request & Response - grisp_spi","doc":"From the controller point of view, requests are always written to the MOSI\nline and response data is always read from the MISO line.\n\nBecause SPI is synchronous, for every request byte sent a response byte is\nalways received. Many responder devices respond silently with zeroes until a\nrequest is fully received, then send their response while ignoring the rest\nof the bytes being received. Other devices can start sending data already\nbefore the controller has finished sending a request, and even reply with\ndata while the controller is writing its own data.\n\nThis module provides two message formats. One format (`t:message_raw/0`)\nreturns the full response, all byaes received from the responder device\nincluding ones sent during the initial request.\n\nThe other format (`t:message_simple/0`) automatically pads the request so\nthat response bytes can be received, and strips the response of an initial\nnumber of bytes. This makes it simpler to communicate with responder devices\nthat wait for a fully received request before replying.","ref":"grisp_spi.html#module-request-response"},{"type":"function","title":"grisp_spi.open/1","doc":"Opens an SPI bus with the default chip select pin.\n\nThe respective pin 1 is used as `DefaultPin` for each slot.\nSee `m:grisp_gpio`.","ref":"grisp_spi.html#open/1"},{"type":"function","title":"grisp_spi.open/2","doc":"Opens an SPI bus.\n\nChip select pin `CS` can be any valid GPIO output pin.\n\nSee `m:grisp_gpio`.","ref":"grisp_spi.html#open/2"},{"type":"function","title":"grisp_spi.transfer/2","doc":"Transfers SPI messages on a bus.\n\nA list of responses is returned in the same order as their respective\nmessages.\n\nThere are two forms of messages, raw binary messages or a simple message with\npadding and skipping.\n\n| `t:message_raw/0`    | A raw binary message `{Mode, Binary}` sends the data and generates a response of the same length.|\n| `t:message_simple/0` | A simple message tuple `{Mode, Binary, Skip, Pad}` additionally contains the amount of bytes to skip in the response and an amount of padding bytes to add to the request. The response length will be the length of the request plus the pad length minus the skip length. |\n\nSee [Request & Response](#module-request-response) for more\ninformation.","ref":"grisp_spi.html#transfer/2"},{"type":"type","title":"grisp_spi.bus/0","doc":"SPI bus identifier.","ref":"grisp_spi.html#t:bus/0"},{"type":"type","title":"grisp_spi.clock/0","doc":"SPI clock configuration.","ref":"grisp_spi.html#t:clock/0"},{"type":"type","title":"grisp_spi.message/0","doc":"SPI message","ref":"grisp_spi.html#t:message/0"},{"type":"type","title":"grisp_spi.message_raw/0","doc":"A message where the request is unmodified and which will yield a response\nbinary of the same size.","ref":"grisp_spi.html#t:message_raw/0"},{"type":"type","title":"grisp_spi.message_simple/0","doc":"A message where the request itself is padded with `Pad` number of `0` bytes,\nand will yield a response binary that is stripped of its first `Skip`\nnumber of bytes.","ref":"grisp_spi.html#t:message_simple/0"},{"type":"type","title":"grisp_spi.mode/0","doc":"SPI transfer mode.","ref":"grisp_spi.html#t:mode/0"},{"type":"opaque","title":"grisp_spi.ref/0","doc":"Reference to an opened SPI bus.","ref":"grisp_spi.html#t:ref/0"},{"type":"type","title":"grisp_spi.response/0","doc":"SPI response.","ref":"grisp_spi.html#t:response/0"},{"type":"module","title":"onewire_ds18b20","doc":"Communicate with the\n[DS18B20 - Programmable Resolution 1-Wire Digital Thermometer](https://datasheets.maximintegrated.com/en/ds/DS18B20.pdf)","ref":"onewire_ds18b20.html"},{"type":"function","title":"onewire_ds18b20.convert/2","doc":"Initiate a temperature measurement.","ref":"onewire_ds18b20.html#convert/2"},{"type":"function","title":"Example - onewire_ds18b20.convert/2","doc":"```\n 1> onewire_ds18b20:convert([40,255,190,25,96,23,3,203], 500).\n ok\n```","ref":"onewire_ds18b20.html#convert/2-example"},{"type":"function","title":"onewire_ds18b20.read_scratchpad/1","doc":"Read the scratchpad.\n\nReturns the two bytes of the temperature register (`LSB` and `MSB`) and\nthe one byte of the configuration register.","ref":"onewire_ds18b20.html#read_scratchpad/1"},{"type":"function","title":"onewire_ds18b20.temp/1","doc":"Read the temperature in °C from the scratchpad.","ref":"onewire_ds18b20.html#temp/1"},{"type":"function","title":"Example - onewire_ds18b20.temp/1","doc":"```\n onewire_ds18b20:temp([40,255,190,25,96,23,3,203]).\n 22.375\n```","ref":"onewire_ds18b20.html#temp/1-example"},{"type":"module","title":"onewire_ds2408","doc":null,"ref":"onewire_ds2408.html"},{"type":"function","title":"onewire_ds2408.read_channel/2","doc":null,"ref":"onewire_ds2408.html#read_channel/2"},{"type":"function","title":"onewire_ds2408.write_channel/2","doc":null,"ref":"onewire_ds2408.html#write_channel/2"},{"type":"module","title":"pmod_acl2","doc":"","ref":"pmod_acl2.html"},{"type":"function","title":"pmod_acl2.g/0","doc":"","ref":"pmod_acl2.html#g/0"},{"type":"function","title":"pmod_acl2.raw/0","doc":"","ref":"pmod_acl2.html#raw/0"},{"type":"module","title":"pmod_als","doc":"Digilent Pmod_ALS module.\n\nThis component provides ambient light-to-digital\nsensing through the SPI2 interface on a 6-pin connector.\n\nFor further information about the Digilent Pmod_ALS\nand its components :\nhttps://store.digilentinc.com/pmod-als-ambient-light-sensor/\n\nTexas Instrument's ADC081S021 analog-to-digital converter\nhttp://www.ti.com/lit/ds/symlink/adc081s021.pdf\n\nVishay Semiconductor's TEMT6000X01.\nhttp://www.vishay.com/docs/81579/temt6000.pdf\n\nStart the driver with\n```\n 1> grisp:add_device(spi2, pmod_als).\n```","ref":"pmod_als.html"},{"type":"function","title":"pmod_als.percentage/0","doc":"Returns a the percentage of current ambient light\nbased on the `read/0` function. The value\nis rounded to the closest integer.","ref":"pmod_als.html#percentage/0"},{"type":"function","title":"pmod_als.read/0","doc":"Returns the ambient light value that is currently sensed\nby the ALS module. On success, the return value is a number\nin the 0..255 range that is proportional to the luminous\nintensity.\n\nTechnically, the values are representative of the\npower perceived by the phototransistor from the light source\nincoming at an angle of ±60°. It is a model of the response\nof the human eye to the same light source that is obtained\nby calculating the luminosity function.\n\nThe peak wavelength sensitivity of the module is at 570nm\nmaking it close to the human eye (555nm). This implies that\nreturn values will be the highest when the ALS is exposed to\nwavelengths of green light with a slight yellow tint.","ref":"pmod_als.html#read/0"},{"type":"module","title":"pmod_dio","doc":null,"ref":"pmod_dio.html"},{"type":"function","title":"pmod_dio.crc5/1","doc":null,"ref":"pmod_dio.html#crc5/1"},{"type":"function","title":"pmod_dio.handle_call/3","doc":null,"ref":"pmod_dio.html#handle_call/3"},{"type":"function","title":"pmod_dio.handle_cast/2","doc":null,"ref":"pmod_dio.html#handle_cast/2"},{"type":"function","title":"pmod_dio.init/1","doc":null,"ref":"pmod_dio.html#init/1"},{"type":"function","title":"pmod_dio.read/2","doc":null,"ref":"pmod_dio.html#read/2"},{"type":"function","title":"pmod_dio.read_burst/2","doc":null,"ref":"pmod_dio.html#read_burst/2"},{"type":"function","title":"pmod_dio.start_link/2","doc":null,"ref":"pmod_dio.html#start_link/2"},{"type":"function","title":"pmod_dio.write/3","doc":null,"ref":"pmod_dio.html#write/3"},{"type":"module","title":"pmod_gps","doc":"[PmodGPS](https://reference.digilentinc.com/reference/pmod/pmodgps/reference-manual)\nmodule.\n\nThe PmodGPS sends the GPS data over UART.\n\nStart the driver with\n```\n1> grisp:add_device(uart, pmod_gps).\n```","ref":"pmod_gps.html"},{"type":"function","title":"pmod_gps.get/1","doc":"Get the GPS data.\n\nThe input parameter specifies which type of sentence to get.\nFor a description of the sentences see the\n[PmodGPS Reference Manual](https://reference.digilentinc.com/_media/reference/pmod/pmodgps/pmodgps_rm.pdf).\n\nThe sentence CRC is checked for all the sentence types, but for now only the\nGGA values are parsed. If other sentences are needed, `m:grisp_nmea` needs to be\nextended to support more types.","ref":"pmod_gps.html#get/1"},{"type":"function","title":"Example - pmod_gps.get/1","doc":"```\n 2> pmod_gps:get(gga).\n {gps,gga,#{alt => 61.9,fixed => true,lat => 52.122661666666666,long => 11.594928333333334,time => 53912000}}}\n 3> pmod_gps:get(gsa).\n {gps,gsa,<<\\\"A,3,17,06,19,02,24,,,,,,,,2.69,2.51,0.97\\\">>}\n 4> pmod_gps:get(gsv).\n {gps,gsv,<<\\\"3,3,12,14,22,317,17,17,10,040,35,29,09,203,,22,02,351\\\">>}\n 5> pmod_gps:get(rmc).\n {gps,rmc,<<\\\"150007.000,A,5207.3592,N,01135.6895,E,0.46,255.74,120220,,,A\\\">>}\n 6> pmod_gps:get(vtg).\n {gps,vtg,<<\\\"297.56,T,,M,0.65,N,1.21,K,A\\\">>}\n```","ref":"pmod_gps.html#get/1-example"},{"type":"module","title":"pmod_gyro","doc":"[PmodGYRO](https://reference.digilentinc.com/reference/pmod/pmodgyro/reference-manual)\nmodule that gets the gyroscopes data via SPI.\n\nStart the driver with\n```\n1> grisp:add_device(spi1, pmod_gyro).\n```","ref":"pmod_gyro.html"},{"type":"function","title":"pmod_gyro.read/0","doc":"Read the gyroscopes X, Y and Z values in degrees per second.","ref":"pmod_gyro.html#read/0"},{"type":"function","title":"Example - pmod_gyro.read/0","doc":"```\n 2> pmod_gyro:read().\n {249.28279313922965,-26.078862235243843,12.764756149667337}\n```","ref":"pmod_gyro.html#read/0-example"},{"type":"module","title":"pmod_hb5","doc":null,"ref":"pmod_hb5.html"},{"type":"function","title":"pmod_hb5.backward/1","doc":null,"ref":"pmod_hb5.html#backward/1"},{"type":"function","title":"pmod_hb5.forward/1","doc":null,"ref":"pmod_hb5.html#forward/1"},{"type":"function","title":"pmod_hb5.open/1","doc":null,"ref":"pmod_hb5.html#open/1"},{"type":"function","title":"pmod_hb5.open/2","doc":null,"ref":"pmod_hb5.html#open/2"},{"type":"function","title":"pmod_hb5.stop/1","doc":null,"ref":"pmod_hb5.html#stop/1"},{"type":"module","title":"pmod_hygro","doc":"API for the\n[PmodHYGRO](https://reference.digilentinc.com/reference/pmod/pmodhygro/start).\n\nStart the driver with\n```\n1> grisp:add_device(i2c, pmod_hygro).\n```","ref":"pmod_hygro.html"},{"type":"function","title":"pmod_hygro.humid/0","doc":"Measure the humidity in %.","ref":"pmod_hygro.html#humid/0"},{"type":"function","title":"Example - pmod_hygro.humid/0","doc":"```\n2> pmod_hygro:humid().\n[{humid,50.225830078125}]\n```","ref":"pmod_hygro.html#humid/0-example"},{"type":"function","title":"pmod_hygro.measurements/0","doc":"Measure the temperature and humidity.","ref":"pmod_hygro.html#measurements/0"},{"type":"function","title":"Example - pmod_hygro.measurements/0","doc":"```\n2> pmod_hygro:measurements().\n[{temp,24.52362060546875},{humid,50.823974609375}]\n```","ref":"pmod_hygro.html#measurements/0-example"},{"type":"function","title":"pmod_hygro.temp/0","doc":"Measure the temperature in °C.","ref":"pmod_hygro.html#temp/0"},{"type":"function","title":"Example - pmod_hygro.temp/0","doc":"```\n2> pmod_hygro:temp().\n[{temp,24.6746826171875}]\n```","ref":"pmod_hygro.html#temp/0-example"},{"type":"module","title":"pmod_maxsonar","doc":"[Pmod MAXSONAR](https://store.digilentinc.com/pmodmaxsonar-maxbotix-ultrasonic-range-finder/)\nmodule.\n\nThe Pmod MAXSONAR cyclically sends measurements via the UART interface.\nThis module converts and stores the latest measurement.\n\nStart the driver with\n```\n1> grisp:add_device(uart, pmod_maxsonar).\n```","ref":"pmod_maxsonar.html"},{"type":"function","title":"pmod_maxsonar.get/0","doc":"Get the latest measured distance in inches.","ref":"pmod_maxsonar.html#get/0"},{"type":"module","title":"pmod_mtds","doc":"GRiSP device driver for Digilent Pmod MTDS.\n\n> **NOTE:** This driver does **not** replicate all of the functionality found\nin the reference C++ driver.  I hope it replicates enough that interested\ndevelopers have templates for any further features they might want to add!\nAlso, a warning: the C++ driver defines some constants (and even commands??)\nwhich are **not** supported by the device firmware.\n\nFor manuals and a C++ reference driver, see\nhttps://github.com/Digilent/vivado-library/tree/master/ip/Pmods/PmodMTDS_v1_0","ref":"pmod_mtds.html"},{"type":"function","title":"pmod_mtds.bitmap/3","doc":"Allocate a new bitmap (usually to back a surface).","ref":"pmod_mtds.html#bitmap/3"},{"type":"function","title":"pmod_mtds.bitmap_draw/5","doc":"Write the contents of one bitmap onto another.","ref":"pmod_mtds.html#bitmap_draw/5"},{"type":"function","title":"pmod_mtds.bitmap_release/1","doc":"Release a bitmap.","ref":"pmod_mtds.html#bitmap_release/1"},{"type":"function","title":"pmod_mtds.clear/1","doc":"Blanks the MTDS to the indicated color.","ref":"pmod_mtds.html#clear/1"},{"type":"function","title":"pmod_mtds.color_bg/2","doc":"Set the background draw color of a surface.","ref":"pmod_mtds.html#color_bg/2"},{"type":"function","title":"pmod_mtds.color_fg/2","doc":"Set the foreground draw color of a surface.","ref":"pmod_mtds.html#color_fg/2"},{"type":"function","title":"pmod_mtds.draw_rop/2","doc":"Set the bitwise operator used to combine pen + bitmap when drawing.","ref":"pmod_mtds.html#draw_rop/2"},{"type":"function","title":"pmod_mtds.font/1","doc":"Retrieve the current font of a surface.","ref":"pmod_mtds.html#font/1"},{"type":"function","title":"pmod_mtds.font/2","doc":"Set the active font for a surface.","ref":"pmod_mtds.html#font/2"},{"type":"function","title":"pmod_mtds.line_to/2","doc":"Draw a line on a surface from the current cursor position to a new one.","ref":"pmod_mtds.html#line_to/2"},{"type":"function","title":"pmod_mtds.move_to/2","doc":"Move the cursor position on a surface.","ref":"pmod_mtds.html#move_to/2"},{"type":"function","title":"pmod_mtds.register/0","doc":"Registers the caller for touch events for the indicated window.","ref":"pmod_mtds.html#register/0"},{"type":"function","title":"pmod_mtds.register/1","doc":"","ref":"pmod_mtds.html#register/1"},{"type":"function","title":"pmod_mtds.start_link/2","doc":"Launch the MTDS driver.","ref":"pmod_mtds.html#start_link/2"},{"type":"function","title":"pmod_mtds.stock_font/1","doc":"Convert a human-readable font atom to a font() instance.","ref":"pmod_mtds.html#stock_font/1"},{"type":"function","title":"pmod_mtds.surface/0","doc":"Generate a new surface (with no underlying bitmap).","ref":"pmod_mtds.html#surface/0"},{"type":"function","title":"pmod_mtds.surface_bitmap/2","doc":"Tether a surface to a backing bitmap.","ref":"pmod_mtds.html#surface_bitmap/2"},{"type":"function","title":"pmod_mtds.surface_display/0","doc":"Retrieve a surface handle to the global display.","ref":"pmod_mtds.html#surface_display/0"},{"type":"function","title":"pmod_mtds.surface_release/1","doc":"Release a surface handle.","ref":"pmod_mtds.html#surface_release/1"},{"type":"function","title":"pmod_mtds.text/3","doc":"Write text on a surface at a given position.","ref":"pmod_mtds.html#text/3"},{"type":"type","title":"pmod_mtds.bitmap/0","doc":"","ref":"pmod_mtds.html#t:bitmap/0"},{"type":"type","title":"pmod_mtds.color/0","doc":"","ref":"pmod_mtds.html#t:color/0"},{"type":"type","title":"pmod_mtds.font/0","doc":"","ref":"pmod_mtds.html#t:font/0"},{"type":"type","title":"pmod_mtds.position/0","doc":"","ref":"pmod_mtds.html#t:position/0"},{"type":"type","title":"pmod_mtds.stock_font/0","doc":"","ref":"pmod_mtds.html#t:stock_font/0"},{"type":"type","title":"pmod_mtds.surface/0","doc":"","ref":"pmod_mtds.html#t:surface/0"},{"type":"type","title":"pmod_mtds.window/0","doc":"","ref":"pmod_mtds.html#t:window/0"},{"type":"module","title":"pmod_nav","doc":"Driver module for the [PmodNAV](https://store.digilentinc.com/pmod-nav-9-axis-imu-plus-barometer/) 9-axis IMU plus barometer device.\n\nFor more information see the Tutorial at the GRiSP Wiki: [PmodNAV Tutorial](https://github.com/grisp/grisp/wiki/PmodNAV-Tutorial).\n\nStart the driver with\n```\n1> grisp:add_device(spi1, pmod_nav).\n```","ref":"pmod_nav.html"},{"type":"function","title":"pmod_nav.config/2","doc":"Change configurations.","ref":"pmod_nav.html#config/2"},{"type":"function","title":"Examples - pmod_nav.config/2","doc":"To switch to accelerometer only mode, i.e., power down the gyroscope, use:\n```\n2> pmod_nav:config(acc, #{odr_g => power_down}).\nok\n```\n\nTo turn the gyroscope back on use:\n```\n3> pmod_nav:config(acc, #{odr_g => {hz,14.9}}).\nok\n```\n\nFor more possible configurations see the datasheets\n[LSM9DS1](http://www.st.com/web/en/resource/technical/document/datasheet/DM00103319.pdf)\nand\n[LPS25HB](http://www.st.com/web/en/resource/technical/document/datasheet/DM00141379.pdf).\n\nUse `registers/0` and `registers/1` to see the mapping between\nErlang expressions and the bits on the hardware.","ref":"pmod_nav.html#config/2-examples"},{"type":"function","title":"pmod_nav.read/2","doc":"","ref":"pmod_nav.html#read/2"},{"type":"function","title":"pmod_nav.read/3","doc":"Read registers of a component.","ref":"pmod_nav.html#read/3"},{"type":"function","title":"Examples - pmod_nav.read/3","doc":"To read the accelerometer X, Y and Z axises G-forces in milli g, use:\n```\n 4> pmod_nav:read(acc, [out_x_xl, out_y_xl, out_z_xl], #{xl_unit => mg}).\n [50.813,6.527,983.7470000000001]\n```","ref":"pmod_nav.html#read/3-examples"},{"type":"function","title":"Further Registers - pmod_nav.read/3","doc":"| Component | Registers                        | Possible Options                               | Description                                                    |\n| --------- | -------------------------------- | ---------------------------------------------- | -------------------------------------------------------------- |\n| `acc`     | `[out_x_g, out_y_g, out_z_g]`    | `#{g_unit => dps \\| mdps} default dps`         | Rotation of the axises x,y and z in (milli) degrees per second |\n| `acc`     | `[out_x_xl, out_y_xl, out_z_xl]` | `#{xl_unit => g \\| mg} default g`              | G-force on the axises x,y and z                                |\n| `mag`     | `[out_x_m, out_y_m, out_z_m]`    | `#{mag_unit => gauss \\| mgauss} default gauss` | Strength of the magnetic field in (milli) gauss                |\n| `alt`     | `[press_out]`                    |                                                | Pressure in hPa                                                |\n| `alt`     | `[temp_out]`                     |                                                | Temperature in °C                                              |\n\nFor all registers see `registers/0` and `registers/1`\nand use the datasheets\n[LPS25HB](http://www.st.com/web/en/resource/technical/document/datasheet/DM00103319.pdf)\nand\n[LPS25HB](http://www.st.com/web/en/resource/technical/document/datasheet/DM00141379.pdf)\nfor a complete description.","ref":"pmod_nav.html#read/3-further-registers"},{"type":"function","title":"pmod_nav.registers/0","doc":"Get the registers (with the possible entries) of all components.","ref":"pmod_nav.html#registers/0"},{"type":"function","title":"pmod_nav.registers/1","doc":"Get the registers (with the possible entries) of one component.","ref":"pmod_nav.html#registers/1"},{"type":"function","title":"Example - pmod_nav.registers/1","doc":"To see the possible configurations in `ctrl_reg1_g` use:\n```\n4> maps:find(ctrl_reg1_g, pmod_nav:registers(acc)).\n{ok,{16,read_write,1,\n     [{odr_g,3,\n             #{power_down => 0,\n               {hz,119} => 3,\n               {hz,238} => 4,\n               {hz,476} => 5,\n               {hz,952} => 6,\n               {hz,14.9} => 1,\n               {hz,59.5} => 2}},\n      {fs_g,2,#{{dps,245} => 0,{dps,500} => 1,{dps,2000} => 3}},\n      {0,1},\n      {bw_g,2,raw}]}}\n```","ref":"pmod_nav.html#registers/1-example"},{"type":"type","title":"pmod_nav.component/0","doc":"","ref":"pmod_nav.html#t:component/0"},{"type":"type","title":"pmod_nav.opts/0","doc":"","ref":"pmod_nav.html#t:opts/0"},{"type":"type","title":"pmod_nav.register/0","doc":"","ref":"pmod_nav.html#t:register/0"},{"type":"module","title":"pmod_ssr","doc":"[PmodSSR](https://digilent.com/reference/pmod/pmodssr/reference-manual) module.\n\nThe Pmod SSR (Solid State Relay) module is designed to control high-power devices using a GPIO pin from a GRiSP board.\n\n| Pin | Function                           |\n| --- | ---------------------------------- |\n| 1   | ON/OFF control (connected to GPIO) |\n| 5   | Ground (GND)                       |\n| 6   | 3.3V Power Supply (VCC)            |\n\nThis relay can be switched on or off by sending a control signal through a single GPIO pin (minimal setup needed for controlling high-power loads).\nThe driver allows you to easily turn the relay on and off through simple commands.\n\n\nStart the driver with\n```\n1> grisp:add_device(gpio1, pmod_ssr).\n```","ref":"pmod_ssr.html"},{"type":"module","title":"Examples - pmod_ssr","doc":"To turn on, use:\n```\n2> pmod_ssr:on().\nok\n```\n\nTo turn back off, use:\n```\n3> pmod_ssr:off().\nok\n```","ref":"pmod_ssr.html#module-examples"},{"type":"function","title":"pmod_ssr.off/0","doc":"","ref":"pmod_ssr.html#off/0"},{"type":"function","title":"pmod_ssr.on/0","doc":"","ref":"pmod_ssr.html#on/0"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.0.0/),\nand this project adheres to\n[Semantic Versioning](https://semver.org/spec/v2.0.0.html).\n\n## [Unreleased]\n\n## [2.8.0] - 2025-02-03","ref":"changelog.html"},{"type":"extras","title":"Added - Changelog","doc":"- Add issue trigger to otp_pkg.yaml [grisp/#147](https://github.com/grisp/grisp/pull/147)\n- Announce new package in Slack [grisp/#148](https://github.com/grisp/grisp/pull/148)\n- Add the module grisp_info to get information about the grisp hardware, the boot status and the current software on system partions or sdcard. [grisp/#149](https://github.com/grisp/grisp/pull/149)\n- Add PWM support [grisp/#153](https://github.com/grisp/grisp/pull/153)\n- Add missing pin in the docs [grisp/#152](https://github.com/grisp/grisp/pull/152)\n- Add MTDS driver [grisp/#151](https://github.com/grisp/grisp/pull/151)","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Port documentation to ExDoc [grisp/#155](https://github.com/grisp/grisp/pull/155)","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fix cached version for CI cron job [grisp/#145](https://github.com/grisp/grisp/pull/145)\n- Fix check_otp_rel.yaml [grisp/#146](https://github.com/grisp/grisp/pull/146)\n- Fix CI/CD package upload\n- Fix duplication of xcomp files in overlay [grisp/#150](https://github.com/grisp/grisp/pull/150)o\n- Fix edoc and check in CI [grisp/#154](https://github.com/grisp/grisp/pull/154)\n\n## [2.7.0] - 2024-10-11","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- New pmod_ssr driver\n- Support for rebooting the board on crash and/or exit of the VM, so calling\n  init:stop() will reboot the board if the option on_exit is set to reboot in\n  grisp.ini.\n\n## [2.6.0] - 2024-07-29","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fix barebox error handling [grisp/#137](https://github.com/grisp/grisp/pull/137)\n- Fix the commands available in RTEMS shell [grisp/#132](https://github.com/grisp/grisp/pull/132)","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Expose emulation status through grisp.hrl macro ?IS_EMULATED [grisp/#137](https://github.com/grisp/grisp/pull/137)\n- Copy configuration files to /etc during boot [grisp/#132](https://github.com/grisp/grisp/pull/132)\n- Add default configuration for DNS support [grisp/#132](https://github.com/grisp/grisp/pull/132)\n- Add RTEMS log priority option in INI file [grisp/#132](https://github.com/grisp/grisp/pull/132)\n\n## [2.5.0] - 2024-06-25","ref":"changelog.html#added"},{"type":"extras","title":"Added - Remove ssl patch for OTP-26 builds - Changelog","doc":"[grisp/#127](https://github.com/grisp/grisp/pull/127) - Add support for OTP 27\n[grisp/#130](https://github.com/grisp/grisp/pull/130) - Add support for OTP 26.2\n[grisp/#131](https://github.com/grisp/grisp/pull/131)\n\n## [2.4.0] - 2024-01-18","ref":"changelog.html#added-remove-ssl-patch-for-otp-26-builds"},{"type":"extras","title":"Added - Changelog","doc":"- Support for OTP 26 [grisp/#121](https://github.com/grisp/grisp/pull/121)\n\n- Add BLAS and LAPACK nif support [grisp/#118](https://github.com/grisp/grisp/pull/118)\n\n- New `shell` option for grisp.ini [grisp/#125](https://github.com/grisp/grisp/pull/125)","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fix emulation for grisp 2 [grisp/#117](https://github.com/grisp/grisp/pull/117)\n\n## [2.3.0] - 2022-07-27","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Support for OTP 25\n\n## [2.2.0] - 2022-07-18","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"- Support for OTP 24","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- Removed redundant command line argument boot debug output\n\n## [2.1.0] - 2022-06-07","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- It is now possible to use UART (lower row) and I2C PMOD ports as GPIO pins\n- PmodHB5 driver now supports the new available GPIO pins\n\n## [2.0.0] - 2022-02-01\n\n**\\*Note! This release is not compatible with GRiSP 1 yet!** GRiSP 1\ncompatibility will get added in a later patch release.\\*","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"- `grisp_hw` module for hardware access functions\n- `grisp_rtems` module containing NIFs for selected RTEMS functions","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- `grisp_gpio` interface has been updated and simplified\n- `grisp_spi` interface has been updated and simplified\n- `grisp_i2c` interface has been updated and simplified\n- `pmod_gps` now validates the NMEA sentences and parses the GGA ones\n- `pmod_gps` API changed, it now returns a pre-parsed tuple, and for the GGA a\n  parsed map of the sentence fields","ref":"changelog.html#changed"},{"type":"extras","title":"Removed - Changelog","doc":"- **Breaking change!** The old GPIO API has been removed in favor of a new API\n- **Breaking change!** The old SPI API has been removed in favor of a new API\n- **Breaking change!** The old I2C API has been removed in favor of a new API\n\n## [1.2.1] - 2020-07-02","ref":"changelog.html#removed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Updated to `grisp_emulation` 0.1.2 with proper PmodGYRO emulation\n  ([\\#76](https://github.com/grisp/grisp/issues/76))\n\n## [1.2.0] - 2020-03-03","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Add link to totorial to Pmod NAV doc [\\#59](https://github.com/grisp/grisp/issues/59)\n- Add documentation [\\#71](https://github.com/grisp/grisp/pull/71) ([maehjam](https://github.com/maehjam))\n\n* Add documentation [\\#68](https://github.com/grisp/grisp/pull/68) ([maehjam](https://github.com/maehjam))\n* Add documentation for PmodNAV [\\#67](https://github.com/grisp/grisp/pull/67) ([maehjam](https://github.com/maehjam))","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- add_device fails for PmodHYGRO [\\#69](https://github.com/grisp/grisp/issues/69)\n\n* Fix for add_device, fix of read message, add documentation [\\#70](https://github.com/grisp/grisp/pull/70) ([maehjam](https://github.com/maehjam))\n* Fix edoc syntax [\\#66](https://github.com/grisp/grisp/pull/66) ([nextl00p](https://github.com/nextl00p))\n\n## [1.1.6] - 2019-09-27","ref":"changelog.html#fixed"},{"type":"extras","title":"Changed - Changelog","doc":"- Remove deprecated maintainers section [\\#65](https://github.com/grisp/grisp/pull/65) ([nextl00p](https://github.com/nextl00p))\n\n## [1.1.5] - 2019-09-27","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Make it possible to configure UART pins as GPIO [\\#37](https://github.com/grisp/grisp/issues/37)\n- Implement read command for pmod_gyro [\\#61](https://github.com/grisp/grisp/pull/61) ([GalaxyGorilla](https://github.com/GalaxyGorilla))\n- Gps [\\#57](https://github.com/grisp/grisp/pull/57) ([aytchell](https://github.com/aytchell))\n- Add support for OTP 22 [\\#54](https://github.com/grisp/grisp/pull/54) ([sylane](https://github.com/sylane))\n- Add missing SPI pins [\\#56](https://github.com/grisp/grisp/pull/56) ([Theuns-Botha](https://github.com/Theuns-Botha))\n- Examples for grisp_led:pattern/2 using functions [\\#50](https://github.com/grisp/grisp/pull/50) ([Laymer](https://github.com/Laymer))\n- Add Feature wireless ad hoc network mode [\\#41](https://github.com/grisp/grisp/pull/41) ([Laymer](https://github.com/Laymer))\n- Add Digilent Pmod_ALS ambient light sensor driver. [\\#40](https://github.com/grisp/grisp/pull/40) ([Laymer](https://github.com/Laymer))\n- Add pmod hygro driver [\\#31](https://github.com/grisp/grisp/pull/31) ([sebb7](https://github.com/sebb7))","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- Edoc doesn't build on master [\\#60](https://github.com/grisp/grisp/issues/60)\n- grisp_led:pattern documentation is missing the Fun argument [\\#49](https://github.com/grisp/grisp/issues/49)\n- Data decoding for pmod_maxsonar is wrong [\\#42](https://github.com/grisp/grisp/issues/42)\n- Quickfix for typo [\\#64](https://github.com/grisp/grisp/pull/64) ([Laymer](https://github.com/Laymer))\n\n* Quickfix decoding pattern [\\#51](https://github.com/grisp/grisp/pull/51) ([Laymer](https://github.com/Laymer))\n* Split out emulation layer [\\#46](https://github.com/grisp/grisp/pull/46) ([Theuns-Botha](https://github.com/Theuns-Botha))\n* Fixed data decoding for 'Digilent PmodMAXSONAR' [\\#43](https://github.com/grisp/grisp/pull/43) ([aytchell](https://github.com/aytchell))\n\n## [1.1.4] - 2018-07-30","ref":"changelog.html#fixed"},{"type":"extras","title":"Fixed - Changelog","doc":"- PmodNAV magnetometer fails initialization \"sometimes\" [\\#11](https://github.com/grisp/grisp/issues/11)\n\n## [1.1.3] - 2018-07-30","ref":"changelog.html#fixed"},{"type":"extras","title":"Changed - Changelog","doc":"- Make embedded mode the default [\\#34](https://github.com/grisp/grisp/pull/34) ([nextl00p](https://github.com/nextl00p))","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- erlang:get_stacktrace/0 is deprecated in OTP 21 [\\#33](https://github.com/grisp/grisp/issues/33)\n- deprecated erlang:get_stacktrace/0 function [\\#36](https://github.com/grisp/grisp/pull/36) ([getong](https://github.com/getong))\n\n## [1.1.2] - 2018-06-21\n\n- Add support for OTP 21.0\n\n## [1.1.1] - 2018-06-06","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Add support for OTP 21.0-rc1 [\\#27](https://github.com/grisp/grisp/pull/27) ([sylane](https://github.com/sylane))\n- \\[WIP\\] Update to support RTEMS 5.0 [\\#26](https://github.com/grisp/grisp/pull/26) ([eproxus](https://github.com/eproxus))","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- Start Erlang runtime when source dependencies are included [\\#25](https://github.com/grisp/grisp/issues/25)\n- Writing to a file hangs the system after listing files [\\#24](https://github.com/grisp/grisp/issues/24)\n\n## [1.1.0] - 2018-05-24","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Add support for DHCP configuration file [\\#23](https://github.com/grisp/grisp/pull/23) ([sylane](https://github.com/sylane))\n- Make onewire driver concurrency safe [\\#21](https://github.com/grisp/grisp/pull/21) ([eproxus](https://github.com/eproxus))\n- Add version to OTP xcomp file [\\#20](https://github.com/grisp/grisp/pull/20) ([sylane](https://github.com/sylane))\n- Add board config and OTP cross-compilation config [\\#18](https://github.com/grisp/grisp/pull/18) ([sylane](https://github.com/sylane))\n\n## [1.0.1] - 2017-12-19","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- Can't compile sample application on Ubuntu 16.04 server [\\#17](https://github.com/grisp/grisp/issues/17)\n- Ubuntu 16.04 is missing build-essential package as dependency [\\#16](https://github.com/grisp/grisp/issues/16)\n- Build failing on Mac 10.10.5 [\\#15](https://github.com/grisp/grisp/issues/15)\n\n## [1.0.0] - 2017-11-17","ref":"changelog.html#fixed"},{"type":"extras","title":"Fixed - Changelog","doc":"- grisp_spi_drv is no gen_server [\\#12](https://github.com/grisp/grisp/issues/12)\n- Fix display of error message during boot, even when correct hostname … [\\#14](https://github.com/grisp/grisp/pull/14) ([nextl00p](https://github.com/nextl00p))\n\n## [0.1.1] - 2017-11-08","ref":"changelog.html#fixed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Added missing files to Hex package\n\n## [0.1.0] - 2017-11-06","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Added emulator instructions in README [\\#8](https://github.com/grisp/grisp/pull/8) ([nextl00p](https://github.com/nextl00p))\n- Quickcheck: model for 3 type of crashes plus clustering [\\#4](https://github.com/grisp/grisp/pull/4) ([ThomasArts](https://github.com/ThomasArts))\n- Quickcheck model for LEDs [\\#1](https://github.com/grisp/grisp/pull/1) ([ThomasArts](https://github.com/ThomasArts))","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Raw refactoring of onewire interface [\\#6](https://github.com/grisp/grisp/pull/6) ([ThomasArts](https://github.com/ThomasArts))","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Supervision restart strategies [\\#3](https://github.com/grisp/grisp/issues/3)\n- grisp_led is accepting interval 0 \\(as well as negative intervals\\) [\\#2](https://github.com/grisp/grisp/issues/2)\n- Fixed wrong registers in rotation vector [\\#9](https://github.com/grisp/grisp/pull/9) ([nextl00p](https://github.com/nextl00p))\n- Fix for \\#2. Negative intervals are now treated by turning off leds [\\#5](https://github.com/grisp/grisp/pull/5) ([nextl00p](https://github.com/nextl00p))\n\n[Unreleased]: https://github.com/grisp/grisp/compare/2.8.0...HEAD\n[2.8.0]: https://github.com/grisp/grisp/compare/2.7.0...2.8.0\n[2.7.0]: https://github.com/grisp/grisp/compare/2.6.0...2.7.0\n[2.6.0]: https://github.com/grisp/grisp/compare/2.5.0...2.6.0\n[2.5.0]: https://github.com/grisp/grisp/compare/2.4.0...2.5.0\n[2.4.0]: https://github.com/grisp/grisp/compare/2.3.0...2.4.0\n[2.3.0]: https://github.com/grisp/grisp/compare/2.2.0...2.3.0\n[2.2.0]: https://github.com/grisp/grisp/compare/2.1.0...2.2.0\n[2.1.0]: https://github.com/grisp/grisp/compare/2.0.0...2.1.0\n[2.0.0]: https://github.com/grisp/grisp/compare/1.2.1...2.0.0\n[1.2.1]: https://github.com/grisp/grisp/compare/1.2.0...1.2.1\n[1.2.0]: https://github.com/grisp/grisp/compare/1.1.6...1.2.0\n[1.1.6]: https://github.com/grisp/grisp/compare/1.1.5...1.1.6\n[1.1.5]: https://github.com/grisp/grisp/compare/1.1.4...1.1.5\n[1.1.4]: https://github.com/grisp/grisp/compare/1.1.3...1.1.4\n[1.1.3]: https://github.com/grisp/grisp/compare/1.1.2...1.1.3\n[1.1.2]: https://github.com/grisp/grisp/compare/1.1.1...1.1.2\n[1.1.1]: https://github.com/grisp/grisp/compare/1.1.0...1.1.1\n[1.1.0]: https://github.com/grisp/grisp/compare/1.0.1...1.1.0\n[1.0.1]: https://github.com/grisp/grisp/compare/1.0.0...1.0.1\n[1.0.0]: https://github.com/grisp/grisp/compare/0.1.1...1.0.0\n[0.1.1]: https://github.com/grisp/grisp/compare/0.1.0...0.1.1\n[0.1.0]: https://github.com/grisp/grisp/compare/09339d122828df2ee9c26338d578519fc084b29b...0.1.0","ref":"changelog.html#fixed"},{"type":"extras","title":"Overview","doc":"Website \n  ·\n   Wiki \n  ·\n   \n    Forums\n   \n \n\n \n   \n \n\nCreate amazing Internet of Things designs without soldering or dropping down to\nC. The GRiSP project makes building internet-connected hardware devices easier\nwith Erlang!\n\n* Real bare-metal Erlang virtual machine (no operating system!)\n* Hard real-time event handling, using open source code\n* Digilent Pmod™ compatible connectors for sensors and actuators\n\nThis repository contains the Erlang runtime and support code for the GRiSP\nhardware platform.\n\n# Getting Started\n\nThe easiest way to get started is to use the [Rebar 3][rebar3_plugin] or\n[Mix][mix_plugin] plug-ins for GRiSP.","ref":"readme.html"},{"type":"extras","title":"Erlang - Overview","doc":"Install the [GRiSP Rebar 3 plug-in][rebar3_plugin] globally by adding `{plugins, [rebar3_grisp]}.` to `~/.config/rebar3/rebar.config`.\n\nCreate a new project:\n\n```sh\n# Create a new GRiSP application (use the real mount path of your SD card)\nrebar3 new grispapp name=demo dest=/path/to/SD-card\ncd demo\n\n# Deploy the application to your SD-card\nrebar3 grisp deploy\n```\n\n# Hardware\n\nGRiSP hardware supports many different connection standards:\n\n* GPIO\n* SPI\n* UART\n* I²C\n* 1-Wire\n\nPeripherals can be connected to either [PMOD] ports or I/O\npins directly.","ref":"readme.html#erlang"},{"type":"extras","title":"GRiSP 2 (`grisp2`) - Overview","doc":"GRiSP 2 ships with the following features:\n\n* 1 × 100 Mbit/s Ethernet port\n* 1 × Wi-Fi 802.11b/g/n WLAN\n* 1 × GPIO PMOD Type 1A (12-pin)\n* 2 × GPIO pins (14-pin & 6-pin)\n* 1 × SPI1 PMOD Type 2 (6-pin)\n* 1 × SPI2 PMOD Type 2A (12-pin)\n* 1 × UART PMOD Type 3A (12-pin)\n* 1 × I²C PMOD Type 6 (6-pin)\n* 1 × I²C Bus (8-pin)\n* 1 × 1-Wire Bus (3-pin)\n* 1 × USB Bus (5-pin)\n* 5 × mode DIP switches\n* 1 × JTAG connector","ref":"readme.html#grisp-2-grisp2"},{"type":"extras","title":"GRiSP 1 (`grisp_base`) - Overview","doc":"GRiSP 1 ships with the following features:\n\n* 1 × Wi-Fi 802.11b/g/n WLAN\n* 2 × GPIO PMOD Type 1 (6-pin)\n* 1 × UART PMOD Type 3 (6-pin)\n* 1 × SPI1 PMOD Type 2A (12-pin)\n* 1 × SPI2 PMOD Type 2 (6-pin)\n* 1 × I²C Bus (8-pin)\n* 1 × 1-Wire Bus (3-pin)\n* 5 × mode DIP switches\n* 1 × JTAG connector\n\n# Testing\n\nThe project has a [hardware emulation layer in software][grisp_emulation] that\nallows you to use the runtime locally on a normal computer. To start a local\nshell for the runtime use:\n\n```\n$ rebar3 as test shell\n```\n\n# Glossary\n\n \n   1-Wire  \n    Long-distance serial communication bus.\n   \n   GPIO  \n    General Purpose Input/Output. Digital signal pin interface used to\n    interface with single pins.\n   \n   I²C  \n    Inter-Intergrated Circuit. Short-distance synchronous serial computer bus.\n   \n   JTAG  \n    On-chip instrumentation and debugging interface.\n   \n   PMOD  \n    A peripheral device that implements the [Digilent Pmod™ connection\n    form factor and interface][PMOD].\n   \n   Slot  \n    A physical slot where a component can be connected. E.g. `SPI1` where an\n    SPI PMOD can be connected.\n   \n   SPI  \n    Serial Peripheral Interface. Synchronous serial communication interface.\n   \n   UART  \n    Universal Asynchronous Receiver-Transmitter. Asynchronous serial\n    communication interface.\n   \n \n\n[PMOD]: https://digilent.com/reference/pmod/start\n[rebar3_plugin]: https://github.com/grisp/rebar3_grisp\n[mix_plugin]: https://github.com/grisp/mix_grisp\n[grisp_emulation]: https://github.com/grisp/grisp_emulation","ref":"readme.html#grisp-1-grisp_base"},{"type":"extras","title":"License","doc":"Apache License\n                           Version 2.0, January 2004\n                        http://www.apache.org/licenses/\n\n   TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION\n\n   1. Definitions.\n\n      \"License\" shall mean the terms and conditions for use, reproduction,\n      and distribution as defined by Sections 1 through 9 of this document.\n\n      \"Licensor\" shall mean the copyright owner or entity authorized by\n      the copyright owner that is granting the License.\n\n      \"Legal Entity\" shall mean the union of the acting entity and all\n      other entities that control, are controlled by, or are under common\n      control with that entity. For the purposes of this definition,\n      \"control\" means (i) the power, direct or indirect, to cause the\n      direction or management of such entity, whether by contract or\n      otherwise, or (ii) ownership of fifty percent (50%) or more of the\n      outstanding shares, or (iii) beneficial ownership of such entity.\n\n      \"You\" (or \"Your\") shall mean an individual or Legal Entity\n      exercising permissions granted by this License.\n\n      \"Source\" form shall mean the preferred form for making modifications,\n      including but not limited to software source code, documentation\n      source, and configuration files.\n\n      \"Object\" form shall mean any form resulting from mechanical\n      transformation or translation of a Source form, including but\n      not limited to compiled object code, generated documentation,\n      and conversions to other media types.\n\n      \"Work\" shall mean the work of authorship, whether in Source or\n      Object form, made available under the License, as indicated by a\n      copyright notice that is included in or attached to the work\n      (an example is provided in the Appendix below).\n\n      \"Derivative Works\" shall mean any work, whether in Source or Object\n      form, that is based on (or derived from) the Work and for which the\n      editorial revisions, annotations, elaborations, or other modifications\n      represent, as a whole, an original work of authorship. For the purposes\n      of this License, Derivative Works shall not include works that remain\n      separable from, or merely link (or bind by name) to the interfaces of,\n      the Work and Derivative Works thereof.\n\n      \"Contribution\" shall mean any work of authorship, including\n      the original version of the Work and any modifications or additions\n      to that Work or Derivative Works thereof, that is intentionally\n      submitted to Licensor for inclusion in the Work by the copyright owner\n      or by an individual or Legal Entity authorized to submit on behalf of\n      the copyright owner. For the purposes of this definition, \"submitted\"\n      means any form of electronic, verbal, or written communication sent\n      to the Licensor or its representatives, including but not limited to\n      communication on electronic mailing lists, source code control systems,\n      and issue tracking systems that are managed by, or on behalf of, the\n      Licensor for the purpose of discussing and improving the Work, but\n      excluding communication that is conspicuously marked or otherwise\n      designated in writing by the copyright owner as \"Not a Contribution.\"\n\n      \"Contributor\" shall mean Licensor and any individual or Legal Entity\n      on behalf of whom a Contribution has been received by Licensor and\n      subsequently incorporated within the Work.\n\n   2. Grant of Copyright License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      copyright license to reproduce, prepare Derivative Works of,\n      publicly display, publicly perform, sublicense, and distribute the\n      Work and such Derivative Works in Source or Object form.\n\n   3. Grant of Patent License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      (except as stated in this section) patent license to make, have made,\n      use, offer to sell, sell, import, and otherwise transfer the Work,\n      where such license applies only to those patent claims licensable\n      by such Contributor that are necessarily infringed by their\n      Contribution(s) alone or by combination of their Contribution(s)\n      with the Work to which such Contribution(s) was submitted. If You\n      institute patent litigation against any entity (including a\n      cross-claim or counterclaim in a lawsuit) alleging that the Work\n      or a Contribution incorporated within the Work constitutes direct\n      or contributory patent infringement, then any patent licenses\n      granted to You under this License for that Work shall terminate\n      as of the date such litigation is filed.\n\n   4. Redistribution. You may reproduce and distribute copies of the\n      Work or Derivative Works thereof in any medium, with or without\n      modifications, and in Source or Object form, provided that You\n      meet the following conditions:\n\n      (a) You must give any other recipients of the Work or\n          Derivative Works a copy of this License; and\n\n      (b) You must cause any modified files to carry prominent notices\n          stating that You changed the files; and\n\n      (c) You must retain, in the Source form of any Derivative Works\n          that You distribute, all copyright, patent, trademark, and\n          attribution notices from the Source form of the Work,\n          excluding those notices that do not pertain to any part of\n          the Derivative Works; and\n\n      (d) If the Work includes a \"NOTICE\" text file as part of its\n          distribution, then any Derivative Works that You distribute must\n          include a readable copy of the attribution notices contained\n          within such NOTICE file, excluding those notices that do not\n          pertain to any part of the Derivative Works, in at least one\n          of the following places: within a NOTICE text file distributed\n          as part of the Derivative Works; within the Source form or\n          documentation, if provided along with the Derivative Works; or,\n          within a display generated by the Derivative Works, if and\n          wherever such third-party notices normally appear. The contents\n          of the NOTICE file are for informational purposes only and\n          do not modify the License. You may add Your own attribution\n          notices within Derivative Works that You distribute, alongside\n          or as an addendum to the NOTICE text from the Work, provided\n          that such additional attribution notices cannot be construed\n          as modifying the License.\n\n      You may add Your own copyright statement to Your modifications and\n      may provide additional or different license terms and conditions\n      for use, reproduction, or distribution of Your modifications, or\n      for any such Derivative Works as a whole, provided Your use,\n      reproduction, and distribution of the Work otherwise complies with\n      the conditions stated in this License.\n\n   5. Submission of Contributions. Unless You explicitly state otherwise,\n      any Contribution intentionally submitted for inclusion in the Work\n      by You to the Licensor shall be under the terms and conditions of\n      this License, without any additional terms or conditions.\n      Notwithstanding the above, nothing herein shall supersede or modify\n      the terms of any separate license agreement you may have executed\n      with Licensor regarding such Contributions.\n\n   6. Trademarks. This License does not grant permission to use the trade\n      names, trademarks, service marks, or product names of the Licensor,\n      except as required for reasonable and customary use in describing the\n      origin of the Work and reproducing the content of the NOTICE file.\n\n   7. Disclaimer of Warranty. Unless required by applicable law or\n      agreed to in writing, Licensor provides the Work (and each\n      Contributor provides its Contributions) on an \"AS IS\" BASIS,\n      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\n      implied, including, without limitation, any warranties or conditions\n      of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A\n      PARTICULAR PURPOSE. You are solely responsible for determining the\n      appropriateness of using or redistributing the Work and assume any\n      risks associated with Your exercise of permissions under this License.\n\n   8. Limitation of Liability. In no event and under no legal theory,\n      whether in tort (including negligence), contract, or otherwise,\n      unless required by applicable law (such as deliberate and grossly\n      negligent acts) or agreed to in writing, shall any Contributor be\n      liable to You for damages, including any direct, indirect, special,\n      incidental, or consequential damages of any character arising as a\n      result of this License or out of the use or inability to use the\n      Work (including but not limited to damages for loss of goodwill,\n      work stoppage, computer failure or malfunction, or any and all\n      other commercial damages or losses), even if such Contributor\n      has been advised of the possibility of such damages.\n\n   9. Accepting Warranty or Additional Liability. While redistributing\n      the Work or Derivative Works thereof, You may choose to offer,\n      and charge a fee for, acceptance of support, warranty, indemnity,\n      or other liability obligations and/or rights consistent with this\n      License. However, in accepting such obligations, You may act only\n      on Your own behalf and on Your sole responsibility, not on behalf\n      of any other Contributor, and only if You agree to indemnify,\n      defend, and hold each Contributor harmless for any liability\n      incurred by, or claims asserted against, such Contributor by reason\n      of your accepting any such warranty or additional liability.\n\n   END OF TERMS AND CONDITIONS\n\n   APPENDIX: How to apply the Apache License to your work.\n\n      To apply the Apache License to your work, attach the following\n      boilerplate notice, with the fields enclosed by brackets \"{}\"\n      replaced with your own identifying information. (Don't include\n      the brackets!)  The text should be enclosed in the appropriate\n      comment syntax for the file format. We also recommend that a\n      file or class name and description of purpose be included on the\n      same \"printed page\" as the copyright notice for easier\n      identification within third-party archives.\n\n   Copyright {yyyy} {name of copyright owner}\n\n   Licensed under the Apache License, Version 2.0 (the \"License\");\n   you may not use this file except in compliance with the License.\n   You may obtain a copy of the License at\n\n       http://www.apache.org/licenses/LICENSE-2.0\n\n   Unless required by applicable law or agreed to in writing, software\n   distributed under the License is distributed on an \"AS IS\" BASIS,\n   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\n   See the License for the specific language governing permissions and\n   limitations under the License.","ref":"license.html"}],"proglang":"erlang","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.35.1"}}